% Code for Sagittal notations with just intonation

tuning = #184858
\include "regular.ly"

%
% Set the accidentals
%

pureStrings = #(map (lambda (entry)
            (cons (* (car entry) (/ 6 grid)) (sagittalString (cdr entry))))
            pureLookup)

pureXExtents = #(map (lambda (entry)
            (cons (* (car entry) (/ 6 grid)) (sagittalXExtent (cdr entry))))
            pureLookup)

pureYExtents = #(map (lambda (entry)
            (cons (* (car entry) (/ 6 grid)) (sagittalYExtent (cdr entry))))
            pureLookup)

mixedStrings = #(map (lambda (entry)
            (cons (* (car entry) (/ 6 grid)) (sagittalString (cdr entry))))
            mixedLookup)

mixedXExtents = #(map (lambda (entry)
            (cons (* (car entry) (/ 6 grid)) (sagittalXExtent (cdr entry))))
            mixedLookup)

mixedYExtents = #(map (lambda (entry)
            (cons (* (car entry) (/ 6 grid)) (sagittalYExtent (cdr entry))))
            mixedLookup)

pure = {
    \override Staff.Accidental #'stencil = #ly:text-interface::print
    \override Staff.Accidental #'text = #(lambda (grob)
        (let ((alteration (ly:grob-property grob 'alteration)))
            (cdr (assoc (quantize alteration) pureStrings))))
    \override Staff.Accidental #'font-name = #"Sagittal"
    \override Staff.Accidental #'font-size = #4
    % c.f. lilypond-user 2014-03-06 or
    % http://code.google.com/p/lilypond/issues/detail?id=2811
    \override Staff.Accidental #'horizontal-skylines = #'()
    \override Staff.Accidental #'X-extent = #(lambda (grob)
        (let ((alteration (ly:grob-property grob 'alteration)))
            (cdr (assoc (quantize alteration) pureXExtents))))
    \override Staff.Accidental #'Y-extent = #(lambda (grob)
        (let ((alteration (ly:grob-property grob 'alteration)))
            (cdr (assoc (quantize alteration) pureYExtents))))
}

mixed = {
    \override Staff.Accidental #'stencil = #ly:text-interface::print
    \override Staff.Accidental #'text = #(lambda (grob)
        (let ((alteration (ly:grob-property grob 'alteration)))
            (cdr (assoc (quantize alteration) mixedStrings))))
    \override Staff.Accidental #'font-name = #"Sagittal"
    \override Staff.Accidental #'font-size = #4
    \override Staff.Accidental #'horizontal-skylines = #'()
    \override Staff.Accidental #'X-extent = #(lambda (grob)
        (let ((alteration (ly:grob-property grob 'alteration)))
            (cdr (assoc (quantize alteration) mixedXExtents))))
    \override Staff.Accidental #'Y-extent = #(lambda (grob)
        (let ((alteration (ly:grob-property grob 'alteration)))
            (cdr (assoc (quantize alteration) mixedYExtents))))
}

%
% JI support
%

% ET is a quantization factor
#(define (raise-by-ratio ET ratio p)
  (let ((reg (ly:pitch-octave p))
        (degree (ly:pitch-notename p))
        (alteration (ly:pitch-alteration p))
        (shift (ratio-tuning ET ratio)))
        (ly:make-pitch reg degree (+ alteration (* 6 shift)))))

#(define tonic-pitch (ly:make-pitch -1 0 0))

#(define (tune-pitch ET ratio p)
    (let* ((relative-tuning (- (oe-pitch ET p) (oe-pitch ET tonic-pitch)))
           (full-shift (- (ratio-tuning ET ratio) relative-tuning))
           (oe-shift (- full-shift (round full-shift))) ; +/- half octave
           (new-alteration (+ (* 6 oe-shift) (ly:pitch-alteration p))))
        (ly:make-pitch
            (ly:pitch-octave p) (ly:pitch-notename p) new-alteration)))

#(define (ratio-tuning ET ratio)
        (/ (inexact->exact (round (* ET (/ (log ratio) (log 2))))) ET))

% equal tempered fraction of an octave
apotome = #(- (* 7 (best-fifth grid)) 4)

% convert from divisions of a 200 cent whole tone
% to quantized divisions of the apotome
tuningFactor = #(/ apotome (ratio-tuning tuning 2187/2048) 6)

% Apply a function to all pitches.
% This is copied from "Transposing music with minimum accidentals"
% In the "Pitches" snippets in the documentation.
#(define (apply-to-pitches fn music)
  (let ((es (ly:music-property music 'elements))
        (e (ly:music-property music 'element))
        (p (ly:music-property music 'pitch)))
    (if (pair? es)
       (ly:music-set-property!
         music 'elements
         (map (lambda (x) (apply-to-pitches fn x)) es)))
    (if (ly:music? e)
       (ly:music-set-property!
         music 'element
         (apply-to-pitches fn e)))
    (if (ly:pitch? p)
       (begin
         (set! p (fn p))
         (ly:music-set-property! music 'pitch p)))
    music))

% The pitch of p in octaves, reduced to between -1/2 and 1/2
#(define (oe-pitch ET p)
    (let ((raw-pitch
            (+ (nominal-tuning ET (ly:pitch-notename p))
               (/ (ly:pitch-alteration p) 6))))
        (- raw-pitch (round raw-pitch))))

% in octaves, ignoring octaves:
#(define (nominal-tuning ET degree)
    (* (best-fifth ET) (vector-ref #(0 2 4 -1 1 3 5) degree)))

tonic = #(define-music-function (parser location m) (ly:music?)
    (begin
        (apply-to-pitches (lambda (p) (begin (set! tonic-pitch p) p))
            (ly:music-deep-copy m))
        (make-music 'SequentialMusic 'void #t)))

% Set the pitch(es) to the given ratio relative to the tonic
tune = #(define-music-function (parser location ratio m)
  (rational? ly:music?)
  (apply-to-pitches (lambda (p) (tune-pitch tuning ratio p))
      (ly:music-deep-copy m)))

% Shift pitch(es) by the desired ratio
raise = #(define-music-function (parser location ratio m)
  (rational? ly:music?)
  (apply-to-pitches (lambda (p) (raise-by-ratio tuning ratio p))
      (ly:music-deep-copy m)))

lower = #(define-music-function (parser location ratio m)
  (rational? ly:music?)
  (apply-to-pitches (lambda (p) (raise-by-ratio tuning (/ ratio) p))
      (ly:music-deep-copy m)))

%
% Cents display
%

centsUp = #(define-music-function (parser location m) (ly:music?)
    (add-cents-to-music 1 m))

centsDown = #(define-music-function (parser location m) (ly:music?)
    (add-cents-to-music -1 m))

#(define (add-cents-to-music direction m)
    (begin
        (event-chord-wrap! m)
        (add-cents-to-chords direction m)))

#(define (add-cents-to-chords direction m)
    (let ((es (ly:music-property m 'elements))
          (e  (ly:music-property m 'element)))
            (if (pair? es)
                (ly:music-set-property! m 'elements
                    (if (eq? (ly:music-property m 'name) 'EventChord)
                        (add-cents-to-chord direction es)
                        (map (lambda (more)
                                (add-cents-to-chords direction more))
                             es))))
            (if (ly:music? e)
                (ly:music-set-property! m 'element
                    (add-cents-to-chords direction e)))
        m))

% Add cents markup to a list including pitches
#(define (add-cents-to-chord direction es)
    (cons (make-music 'TextScriptEvent 'direction direction 'text
            (markup
                #:tiny
                (#:override '(baseline-skip . 1.5)
                    (make-center-column-markup
                        (map format-cents (cents-for-chord
                            (sort
                                (filter (lambda (e) (ly:pitch?
                                        (ly:music-property e 'pitch)))
                                    es)
                                pitch-less)))))))
            es))

% Return the cents deviations from 12-equal for a list of pitches.
#(define (cents-for-chord es)
    (if (null? es) '()
        (let* ((p (ly:music-property (car es) 'pitch))
               (others (cents-for-chord (cdr es)))
               (alter (ly:pitch-alteration p))
               ; ET alteration corresponding to the conventional part
               ; of mixed sagittal
               (etalter (et-alteration alter))
               ; Relative to the conventional part of mixed Sagittal
               (altcents (* 200 (- alter etalter)))
               (nominal-pitch (* 12 (-
                    (oe-pitch 1200 (ly:make-pitch 0 (ly:pitch-steps p) 0))
                    (oe-pitch 1200 tonic-pitch))))
               ; The correction for the nominals not being in equal temperament
               (correction (* 100 (- nominal-pitch (round nominal-pitch)))))
            (cons (+ correction altcents) others))))

% The corresponding alteration for equal temperament
#(define (et-alteration alt)
    (cdr (or
        (assoc (car (reverse
                (assoc (* grid 1/6 (quantize alt)) mixedLookup)))
               '((natural . 0) (sharp . 1/2) (flat . -1/2)
                 (doublesharp . 1) (doubleflat . -1)))
        '(other . 0))))

% Format a number of cents (string of nearest integer)
#(define (format-cents c)
    (string-append (if (< c 0) "" "+")
        (number->string (inexact->exact (round c)))))

% Decide which pitch is lesser for purposes of sorting.
% Takes staff position first (to match visual appearance)
% then alteration as a tie breaker.
#(define (pitch-less ea eb)
    (let* ((pa (ly:music-property ea 'pitch))
           (pb (ly:music-property eb 'pitch))
           (steps-diff (- (ly:pitch-steps pa) (ly:pitch-steps pb))))
        (cond ((< 0 steps-diff) #t)
              ((> 0 steps-diff) #f)
              (<= (ly:pitch-alteration pa) (ly:pitch-alteration pb)))))
